==> currents-api-types-0.0.47.tgz: package/payout/isomorphism.ts <==
import type { Isomorphism } from '../utils/types';
import type {
  PayoutBatchLineItemCompositeKey,
  PayoutBatchItemCompositeKeySerialized,
  PayoutBatchItemLineableCompositeKeySerialized,
  PayoutMethodOriginTypeLineGranular,
  PayoutMethodOriginTypeNonLineGranular,
  PayoutBatchItemNonLineableCompositeKeySerialized,
  PayoutBatchKey,
} from './types';
import type { UserId } from '../user/types';
import type { PayoutMethodOriginType } from '../payoutMethod/types';
import type { Currency } from '../currencies/types';
import {
  payoutBatchItemCompositeKeyLineGranularParser,
  payoutBatchItemCompositeKeyNonLineGranularParser,
  payoutBatchItemCompositeKeyParser,
  payoutBatchKeyParser,
} from './parsers.common';
import { PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES, PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES } from './constants';
import { Option } from 'fp-ts/Option';
import { pipe } from 'fp-ts/function';
import * as O from 'fp-ts/Option';

export const SEPARATOR = ':' as const;

export const payoutBatchItemCompositeKeyIsomorphismFromParseArgs =
  (strict = true) =>
  (serialized: `${PayoutBatchItemCompositeKeySerialized}${string}`) => {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    const [userId, originType, currency, ...rest] = serialized.split(SEPARATOR) as [
      UserId,
      PayoutMethodOriginType,
      Currency,
      string
    ];
    if (strict && rest.length > 0)
      throw new Error(
        `invalid serialized PayoutBatchItemCompositeKey: ${serialized}; extra parts: ${rest.join(SEPARATOR)}`
      );
    return {
      userId,
      originType,
      currency,
    };
  };

export const parser = payoutBatchItemCompositeKeyParser;
export const parserLineable = payoutBatchItemCompositeKeyLineGranularParser;
export const parserNonLineable = payoutBatchItemCompositeKeyNonLineGranularParser;

export const shelfCompositeKeyIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey,
  PayoutBatchItemCompositeKeySerialized
> = {
  to: ({ userId, originType, currency }) => `${userId}${SEPARATOR}${originType}${SEPARATOR}${currency}`,
  from: (o) => parser.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const payoutBatchItemCompositeKeyIsomorphism = shelfCompositeKeyIsomorphism;

export const payoutBatchItemCompositeKeyLineableIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey<PayoutMethodOriginTypeLineGranular>,
  PayoutBatchItemLineableCompositeKeySerialized
> = {
  to: (o) => {
    if (!PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES.includes(o.originType))
      throw new Error(`invalid origin to present as a line granular key ${o.originType}`);
    return payoutBatchItemCompositeKeyIsomorphism.to(o) as PayoutBatchItemLineableCompositeKeySerialized;
  },
  from: (o) => parserLineable.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const payoutBatchItemCompositeKeyNonLineableIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey<PayoutMethodOriginTypeNonLineGranular>,
  PayoutBatchItemNonLineableCompositeKeySerialized
> = {
  to: (o) => {
    if (!PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES.includes(o.originType))
      throw new Error(`invalid origin to present as a line granular key ${o.originType}`);
    return payoutBatchItemCompositeKeyIsomorphism.to(o) as PayoutBatchItemNonLineableCompositeKeySerialized;
  },
  from: (o) => parserNonLineable.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START = '{' as const;
export type PaypalMassPayoutLineNotePrefixStart = typeof PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START;
export const PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END = '}' as const;
export type PaypalMassPayoutLineNotePrefixEnd = typeof PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END;
type PaypalMassPayoutLineNotePrefix =
  `${PaypalMassPayoutLineNotePrefixStart}${PayoutBatchKey}${PaypalMassPayoutLineNotePrefixEnd}`;
export type PaypalMassPayoutLineNote = `${PaypalMassPayoutLineNotePrefix}${string}`;

export const payoutBatchKeyToPaypalMassPayoutLineNotePrefix = (
  payoutBatchKey: PayoutBatchKey
): PaypalMassPayoutLineNotePrefix =>
  `${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START}${payoutBatchKey}${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END}` as const;

export const paypalMassPayoutLineNotePrefixRegexS = `${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START}(.+)${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END}`;

export const payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegexStrict = RegExp(
  `^${paypalMassPayoutLineNotePrefixRegexS}$`
);
export const payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegex = RegExp(`${paypalMassPayoutLineNotePrefixRegexS}`);

export const payoutBatchKeyFromPaypalMassPayoutLineNotePrefix = (prefix: PaypalMassPayoutLineNote): PayoutBatchKey => {
  const match = prefix.match(payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegex);
  if (!match) {
    throw new Error(`panic! Invalid prefix ${prefix}`);
  }
  return payoutBatchKeyParser.parse(match[1]);
};

// could be notes without it, therefore Option
export const extractPaypalMassPayoutLineNotePrefixFromPaypalMassPayoutLineNote = (
  lineNote: PaypalMassPayoutLineNote
): Option<PaypalMassPayoutLineNotePrefix> =>
  pipe(
    lineNote.match(`^${paypalMassPayoutLineNotePrefixRegexS}`),
    O.fromNullable,
    O.map((m) => m[0] as PaypalMassPayoutLineNotePrefix)
  );

export const extractPayoutBatchKeyFromPaypalMassPayoutLineNote = (
  note: PaypalMassPayoutLineNote
): Option<PayoutBatchKey> =>
  pipe(
    note,
    extractPaypalMassPayoutLineNotePrefixFromPaypalMassPayoutLineNote,
    O.map(payoutBatchKeyFromPaypalMassPayoutLineNotePrefix)
  );

export const paypalMassPayoutLineNoteFullRegexS = `^${paypalMassPayoutLineNotePrefixRegexS}(.*)$`;

// has to be a line note; we encode PayoutBatchKey in each of them (note there is necessary redundancy)
export const makePaypalMassPayoutLineNote =
  (key: PayoutBatchKey) =>
  (note = ''): PaypalMassPayoutLineNote => {
    // check the note doesn't start with key, runtime
    const existingBatchKey = extractPayoutBatchKeyFromPaypalMassPayoutLineNote(note as PaypalMassPayoutLineNote);
    if (O.isSome(existingBatchKey)) {
      if (existingBatchKey.value !== key)
        throw new Error(`panic! Note already has a different batch key ${existingBatchKey.value} vs ${key}`);
      return note as PaypalMassPayoutLineNote;
    }
    return `${payoutBatchKeyToPaypalMassPayoutLineNotePrefix(key)}${note}` as PaypalMassPayoutLineNote;
  };

==> currents-api-types-0.0.68.tgz: package/payout/isomorphism.ts <==
import type { Isomorphism } from '../utils/types';
import type {
  PayoutBatchItemCompositeKeySerialized,
  PayoutBatchItemLineableCompositeKeySerialized,
  PayoutBatchItemNonLineableCompositeKeySerialized,
  PayoutBatchKey,
  PayoutBatchLineItemCompositeKey,
  PayoutMethodOriginTypeLineGranular,
  PayoutMethodOriginTypeNonLineGranular,
} from './types';
import type { UserId } from '../user/types';
import type { PayoutMethodOriginType } from '../payoutMethod/types';
import type { Currency } from '../currencies/types';
import {
  payoutBatchItemCompositeKeyLineGranularParser,
  payoutBatchItemCompositeKeyNonLineGranularParser,
  payoutBatchItemCompositeKeyParser,
  payoutBatchKeyParser,
} from './parsers.common';
import { PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES, PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES } from './constants';

export const SEPARATOR = ':' as const;

export const payoutBatchItemCompositeKeyIsomorphismFromParseArgs =
  (strict = true) =>
  (serialized: `${PayoutBatchItemCompositeKeySerialized}${string}`) => {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    const [userId, originType, currency, ...rest] = serialized.split(SEPARATOR) as [
      UserId,
      PayoutMethodOriginType,
      Currency,
      string
    ];
    if (strict && rest.length > 0)
      throw new Error(
        `invalid serialized PayoutBatchItemCompositeKey: ${serialized}; extra parts: ${rest.join(SEPARATOR)}`
      );
    return {
      userId,
      originType,
      currency,
    };
  };

export const parser = payoutBatchItemCompositeKeyParser;
export const parserLineable = payoutBatchItemCompositeKeyLineGranularParser;
export const parserNonLineable = payoutBatchItemCompositeKeyNonLineGranularParser;

export const shelfCompositeKeyIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey,
  PayoutBatchItemCompositeKeySerialized
> = {
  to: ({ userId, originType, currency }) => `${userId}${SEPARATOR}${originType}${SEPARATOR}${currency}`,
  from: (o) => parser.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const payoutBatchItemCompositeKeyIsomorphism = shelfCompositeKeyIsomorphism;

export const payoutBatchItemCompositeKeyLineableIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey<PayoutMethodOriginTypeLineGranular>,
  PayoutBatchItemLineableCompositeKeySerialized
> = {
  to: (o) => {
    if (!PAYOUT_QUEUE_LINE_GRANULAR_ORIGIN_TYPES.includes(o.originType))
      throw new Error(`invalid origin to present as a line granular key ${o.originType}`);
    return payoutBatchItemCompositeKeyIsomorphism.to(o) as PayoutBatchItemLineableCompositeKeySerialized;
  },
  from: (o) => parserLineable.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const payoutBatchItemCompositeKeyNonLineableIsomorphism: Isomorphism<
  PayoutBatchLineItemCompositeKey<PayoutMethodOriginTypeNonLineGranular>,
  PayoutBatchItemNonLineableCompositeKeySerialized
> = {
  to: (o) => {
    if (!PAYOUT_QUEUE_NON_LINE_GRANULAR_ORIGIN_TYPES.includes(o.originType))
      throw new Error(`invalid origin to present as a line granular key ${o.originType}`);
    return payoutBatchItemCompositeKeyIsomorphism.to(o) as PayoutBatchItemNonLineableCompositeKeySerialized;
  },
  from: (o) => parserNonLineable.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(o)),
};

export const PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START = '{' as const;
export type PaypalMassPayoutLineNotePrefixStart = typeof PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START;
export const PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END = '}' as const;
export type PaypalMassPayoutLineNotePrefixEnd = typeof PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END;
export type PaypalMassPayoutLineNotePrefix =
  `${PaypalMassPayoutLineNotePrefixStart}${PayoutBatchKey}${PaypalMassPayoutLineNotePrefixEnd}`;
export type PaypalMassPayoutLineNote = `${PaypalMassPayoutLineNotePrefix}${string}`;

export const payoutBatchKeyToPaypalMassPayoutLineNotePrefix = (
  payoutBatchKey: PayoutBatchKey
): PaypalMassPayoutLineNotePrefix =>
  `${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START}${payoutBatchKey}${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END}` as const;

export const paypalMassPayoutLineNotePrefixRegexS = `${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_START}(.+)${PAYPAL_MASS_PAYOUT_LINE_NOTE_PREFIX_END}`;

export const payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegexStrict = RegExp(
  `^${paypalMassPayoutLineNotePrefixRegexS}$`
);
export const payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegex = RegExp(`${paypalMassPayoutLineNotePrefixRegexS}`);

export const payoutBatchKeyFromPaypalMassPayoutLineNotePrefix = (prefix: PaypalMassPayoutLineNote): PayoutBatchKey => {
  const match = prefix.match(payoutBatchKeyToPaypalMassPayoutLineNotePrefixRegex);
  if (!match) {
    throw new Error(`panic! Invalid prefix ${prefix}`);
  }
  return payoutBatchKeyParser.parse(match[1]);
};

export const paypalMassPayoutLineNoteFullRegexS = `^${paypalMassPayoutLineNotePrefixRegexS}(.*)$`;
